@@ -49,10 +49,21 @@ public:
 
 private:
 
+    int m_mode;
+    wxNotebook* m_about_notebook;
+    wxStaticBitmap * m_logoImgCtrl;
+    wxBitmap m_logo;
+
     DECLARE_EVENT_TABLE()
 
     /**  button to fill the space */
     void OnAboutMe(wxCommandEvent & e);
+    /** retrieves the system informations */
+    void GetSystemInformation(wxFont *font);
+    /** event handler for changing tab */
+    void OnChangedTab(wxNotebookEvent &e);
+    /** function to switch the logo image based on selected tab */
+    void SetMode(int newMode);
 
 
 };
