@@ -59,7 +59,11 @@ using namespace hugin_utils;
 // definition of the control point event
 
 IMPLEMENT_DYNAMIC_CLASS( CPEvent, wxEvent )
+#if _WINDOWS && defined Hugin_shared
+DEFINE_LOCAL_EVENT_TYPE( EVT_CPEVENT )
+#else
 DEFINE_EVENT_TYPE( EVT_CPEVENT )
+#endif
 
 CPEvent::CPEvent( )
 {
@@ -229,7 +233,7 @@ void CPImageCtrl::OnDraw(wxDC & dc)
 {
     wxSize vSize = GetClientSize();
     // draw image (FIXME, redraw only visible regions.)
-    if (editState != NO_IMAGE) {
+    if (editState != NO_IMAGE && m_img.get()) {
 		//clear the blank rectangle to the left of the image
         if (bitmap.GetWidth() < vSize.GetWidth()) {
             dc.SetPen(wxPen(GetBackgroundColour(), 1, wxSOLID));
@@ -361,6 +365,10 @@ wxRect CPImageCtrl::drawPoint(wxDC & dc, const FDiff2D & pointIn, int i, bool se
                    roundi(pointIn.x+l), roundi(pointIn.y+l));
         // only use part inside.
         box &= vigra::Rect2D(m_img->image8->size());
+        if(box.width()<=0 || box.height()<=0)
+        {
+            return wxRect(0,0,0,0);
+        };
         // calculate mean "luminance value"
         vigra::FindAverage<vigra::UInt8> average;   // init functor
         vigra::RGBToGrayAccessor<vigra::RGBValue<vigra::UInt8> > lumac;
@@ -646,25 +654,40 @@ void CPImageCtrl::setImage(const std::string & file, ImageRotation imgRot)
     imageFilename = file;
     wxString fn(imageFilename.c_str(),HUGIN_CONV_FILENAME);
     if (wxFileName::FileExists(fn)) {
-        m_img = ImageCache::getInstance().getImage(imageFilename);
-        editState = NO_SELECTION;
         m_imgRotation = imgRot;
-        rescaleImage();
+        m_img = ImageCache::getInstance().getImageIfAvailable(imageFilename);
+        editState = NO_SELECTION;
+        if (m_img.get()) {
+            rescaleImage();
+        } else {
+            // load the image in the background.
+            m_imgRequest = ImageCache::getInstance().requestAsyncImage(imageFilename);
+            m_imgRequest->ready.connect(
+                boost::bind(&CPImageCtrl::OnImageLoaded, this, _1, _2, _3));
+            // With m_img.get() 0, everything will act as normal except drawing.
+        }
     } else {
         editState = NO_IMAGE;
         bitmap = wxBitmap();
-        SetSizeHints(0,0,0,0,1,1);
         // delete the image (release shared_ptr)
         // create an empty image.
         m_img = ImageCache::EntryPtr(new ImageCache::Entry);
     }
 }
 
-
+void CPImageCtrl::OnImageLoaded(ImageCache::EntryPtr entry, std::string filename, bool load_small)
+{
+    // check we are still displaying this image
+    if (imageFilename == filename)
+    {
+        m_img = entry;
+        rescaleImage();
+    }
+}
 
 void CPImageCtrl::rescaleImage()
 {
-    if (editState == NO_IMAGE) {
+    if (editState == NO_IMAGE || !m_img.get()) {
         return;
     }
     wxImage img = imageCacheEntry2wxImage(m_img);
@@ -855,6 +878,7 @@ CPImageCtrl::EditorState CPImageCtrl::isOccupied(wxPoint mousePos, const FDiff2D
 
 void CPImageCtrl::mouseMoveEvent(wxMouseEvent& mouse)
 {
+    if (!m_img.get()) return; // ignore events if no image loaded.
     wxPoint mpos_;
     CalcUnscrolledPosition(mouse.GetPosition().x, mouse.GetPosition().y,
                            &mpos_.x, & mpos_.y);
@@ -964,6 +988,7 @@ void CPImageCtrl::mouseMoveEvent(wxMouseEvent& mouse)
 void CPImageCtrl::mousePressLMBEvent(wxMouseEvent& mouse)
 {
     DEBUG_DEBUG("LEFT MOUSE DOWN");
+    if (!m_img.get()) return; // ignore events if no image loaded.
     wxPoint mpos_;
     CalcUnscrolledPosition(mouse.GetPosition().x, mouse.GetPosition().y,
                            &mpos_.x, & mpos_.y);
@@ -1009,6 +1034,7 @@ void CPImageCtrl::mousePressLMBEvent(wxMouseEvent& mouse)
 
 void CPImageCtrl::OnTimer(wxTimerEvent & e)
 {
+    if (!m_img.get()) return; // ignore events if no image loaded.
     m_forceMagnifier = false;
     update();
 }
@@ -1016,6 +1042,7 @@ void CPImageCtrl::OnTimer(wxTimerEvent & e)
 void CPImageCtrl::mouseReleaseLMBEvent(wxMouseEvent& mouse)
 {
     DEBUG_DEBUG("LEFT MOUSE UP");
+    if (!m_img.get()) return; // ignore events if no image loaded.
     m_timer.Start(2000, true);
 
     wxPoint mpos_;
@@ -1105,6 +1132,7 @@ void CPImageCtrl::mouseReleaseMMBEvent(wxMouseEvent& mouse)
 void CPImageCtrl::mousePressMMBEvent(wxMouseEvent& mouse)
 {
     DEBUG_DEBUG("middle mouse button pressed, entering scroll mode")
+    if (!m_img.get()) return; // ignore events if no image loaded.
     m_mouseScrollPos = mouse.GetPosition();
 //    SetCursor(wxCursor(wxCURSOR_HAND));
 }
@@ -1112,6 +1140,7 @@ void CPImageCtrl::mousePressMMBEvent(wxMouseEvent& mouse)
 
 void CPImageCtrl::mouseReleaseRMBEvent(wxMouseEvent& mouse)
 {
+    if (!m_img.get()) return; // ignore events if no image loaded.
     wxPoint mpos_;
     CalcUnscrolledPosition(mouse.GetPosition().x, mouse.GetPosition().y,
                            &mpos_.x, & mpos_.y);
@@ -1226,6 +1255,7 @@ void CPImageCtrl::OnSize(wxSizeEvent &e)
 
 void CPImageCtrl::OnKey(wxKeyEvent & e)
 {
+    if (!m_img.get()) return; // ignore events if no image loaded.
     DEBUG_TRACE(" OnKey, key:" << e.m_keyCode);
     wxPoint delta(0,0);
     // check for cursor keys, if control is not pressed
@@ -1331,6 +1361,7 @@ void CPImageCtrl::OnKey(wxKeyEvent & e)
 void CPImageCtrl::OnKeyDown(wxKeyEvent & e)
 {
     DEBUG_TRACE("key:" << e.m_keyCode);
+    if (!m_img.get()) return; // ignore events if no image loaded.
 #if 0
     if (e.m_keyCode == WXK_SHIFT) {
         DEBUG_DEBUG("shift down");
