@@ -81,7 +81,11 @@ private:
 typedef void (wxEvtHandler::*CPEventFunction)(CPEvent&);
 
 BEGIN_DECLARE_EVENT_TYPES()
+#if _WINDOWS && defined Hugin_shared
+    DECLARE_LOCAL_EVENT_TYPE(EVT_CPEVENT,1)
+#else
     DECLARE_EVENT_TYPE(EVT_CPEVENT,1)
+#endif
 END_DECLARE_EVENT_TYPES()
 
 #define EVT_CPEVENT(func) \
@@ -195,6 +199,8 @@ protected:
     wxRect drawPoint(wxDC & p, const hugin_utils::FDiff2D & point, int i, bool selected = false) const;
     // draw the magnified view of a selected control point
     wxBitmap generateMagBitmap(hugin_utils::FDiff2D point, wxPoint canvasPos) const;
+    // display the image when loading finishes
+    void OnImageLoaded(ImageCache::EntryPtr entry, std::string filename, bool load_small);
     void OnDraw(wxDC& dc);
     void OnSize(wxSizeEvent & e);
     void OnKey(wxKeyEvent & e);
@@ -423,6 +429,7 @@ private:
     ImageRotation m_imgRotation;
 
     ImageCache::EntryPtr m_img;
+    ImageCache::RequestPtr m_imgRequest;
 
     bool m_mouseInWindow;
     bool m_forceMagnifier;
