@@ -37,7 +37,6 @@
 #include <GL/glew.h>
 
 #include "panoinc_WX.h"
-
 #include "panoinc.h"
 
 #include "base_wx/platform.h"
@@ -70,6 +69,9 @@ extern "C" {
 #include "PreviewLayoutLinesTool.h"
 
 #include <wx/progdlg.h>
+#if wxCHECK_VERSION(2, 9, 1)
+#include <wx/infobar.h>
+#endif
 
 using namespace utils;
 
@@ -85,7 +87,8 @@ enum {
     ID_SHOW_ALL = wxID_HIGHEST+1711,
     ID_SHOW_NONE = wxID_HIGHEST+1712,
     ID_UNDO = wxID_HIGHEST+1713,
-    ID_REDO = wxID_HIGHEST+1714
+    ID_REDO = wxID_HIGHEST+1714,
+    ID_HIDE_HINTS = wxID_HIGHEST+1715
 };
 
 /** enum, which contains all different toolbar modes */
@@ -112,7 +115,7 @@ BEGIN_EVENT_TABLE(GLPreviewFrame, wxFrame)
     EVT_BUTTON(XRCID("preview_autocrop_tool"), GLPreviewFrame::OnAutocrop)
     EVT_NOTEBOOK_PAGE_CHANGED(XRCID("mode_toolbar_notebook"), GLPreviewFrame::OnSelectMode)
     EVT_NOTEBOOK_PAGE_CHANGING(XRCID("mode_toolbar_notebook"), GLPreviewFrame::OnToolModeChanging)
-   
+    EVT_BUTTON(ID_HIDE_HINTS, GLPreviewFrame::OnHideProjectionHints)
     EVT_BUTTON(XRCID("exposure_default_button"), GLPreviewFrame::OnDefaultExposure)
     EVT_SPIN_DOWN(XRCID("exposure_spin"), GLPreviewFrame::OnDecreaseExposure)
     EVT_SPIN_UP(XRCID("exposure_spin"), GLPreviewFrame::OnIncreaseExposure)
@@ -161,7 +164,7 @@ void AddLabelToBitmapButton(wxBitmapButton* button, wxString new_label,bool Text
     int new_height=0;
     int text_height=0;
     int text_width=0;
-    button->GetTextExtent(new_label, &text_width,&text_height);
+    button->GetTextExtent(new_label.append(wxT(" ")), &text_width,&text_height);
     if(TextBelow)
     {
         new_height=23+text_height;
@@ -201,12 +204,16 @@ void AddLabelToBitmapButton(wxBitmapButton* button, wxString new_label,bool Text
     button->Refresh();
 };
 
-#define PF_STYLE (wxMAXIMIZE_BOX | wxRESIZE_BORDER | wxSYSTEM_MENU | wxCAPTION | wxCLOSE_BOX | wxCLIP_CHILDREN)
+#define PF_STYLE (wxMAXIMIZE_BOX | wxMINIMIZE_BOX | wxRESIZE_BORDER | wxSYSTEM_MENU | wxCAPTION | wxCLOSE_BOX | wxCLIP_CHILDREN)
 #include <iostream>
 GLPreviewFrame::GLPreviewFrame(wxFrame * frame, PT::Panorama &pano)
     : wxFrame(frame,-1, _("Fast Panorama preview"), wxDefaultPosition, wxDefaultSize,
               PF_STYLE),
       m_pano(pano)
+#if !wxCHECK_VERSION(2, 9, 1)
+    ,
+      m_projectionStatusPushed(false)
+#endif
 {
 	DEBUG_TRACE("");
 
@@ -267,6 +274,13 @@ GLPreviewFrame::GLPreviewFrame(wxFrame * frame, PT::Panorama &pano)
     AddLabelToBitmapButton(select_none,_("None"), false);
     m_topsizer->Add(tool_panel, 0, wxEXPAND | wxALL, 2);
     m_topsizer->Add(m_ToggleButtonSizer, 0, wxEXPAND | wxADJUST_MINSIZE | wxBOTTOM, 5);
+    
+#if wxCHECK_VERSION(2, 9, 1)
+    m_infoBar = new wxInfoBar(this);
+    m_infoBar->AddButton(ID_HIDE_HINTS,_("Hide"));
+    m_infoBar->Connect(ID_HIDE_HINTS,wxEVT_COMMAND_BUTTON_CLICKED,wxCommandEventHandler(GLPreviewFrame::OnHideProjectionHints),NULL,this);
+    m_topsizer->Add(m_infoBar, 0, wxEXPAND);
+#endif
 
     wxFlexGridSizer * flexSizer = new wxFlexGridSizer(2,0,5,5);
     flexSizer->AddGrowableCol(0);
@@ -362,10 +376,12 @@ GLPreviewFrame::GLPreviewFrame(wxFrame * frame, PT::Panorama &pano)
     m_BlendModeChoice->Append(_("normal"));
     m_BlendModeChoice->SetSelection(0);
 
-	m_DragModeChoice = XRCCTRL(*this, "drag_mode_choice", wxChoice);
-	m_DragModeChoice->Append(_("normal"));
-	m_DragModeChoice->Append(_("mosaic"));
-	m_DragModeChoice->SetSelection(0);
+    m_DragModeChoice = XRCCTRL(*this, "drag_mode_choice", wxChoice);
+    m_DragModeChoice->Append(_("normal"));
+    m_DragModeChoice->Append(_("mosaic"));
+    m_DragModeChoice->SetSelection(0);
+    // default drag mode
+    GLPreviewFrame::DragChoiceLayout(0);
 
     // TODO implement hdr display in OpenGL, if possible?
     // Disabled until someone can figure out HDR display in OpenGL.
@@ -443,9 +459,9 @@ GLPreviewFrame::GLPreviewFrame(wxFrame * frame, PT::Panorama &pano)
 
     // set the minimize icon
 #ifdef __WXMSW__
-    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/icon.ico"),wxBITMAP_TYPE_ICO);
+    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/hugin.ico"),wxBITMAP_TYPE_ICO);
 #else
-    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/icon.png"),wxBITMAP_TYPE_PNG);
+    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/hugin.png"),wxBITMAP_TYPE_PNG);
 #endif
     SetIcon(myIcon);
 
@@ -461,10 +477,11 @@ GLPreviewFrame::GLPreviewFrame(wxFrame * frame, PT::Panorama &pano)
     if (config->Read(wxT("/GLPreviewFrame/isShown"), 0l) != 0) {
         Show();
     }
+    m_showProjectionHints = config->Read(wxT("/GLPreviewWindow/ShowProjectionHints"), HUGIN_SHOW_PROJECTION_HINTS) == 1;
     wxAcceleratorEntry entries[3];
     entries[0].Set(wxACCEL_NORMAL,WXK_F11,ID_FULL_SCREEN);
     entries[1].Set(wxACCEL_CTRL,(int)'Z',ID_UNDO);
-    entries[2].Set(wxACCEL_CTRL,(int)'Y',ID_REDO);
+    entries[2].Set(wxACCEL_CTRL,(int)'R',ID_REDO);
     wxAcceleratorTable accel(3, entries);
     SetAcceleratorTable(accel);
 #ifdef __WXGTK__
@@ -505,6 +522,10 @@ GLPreviewFrame::~GLPreviewFrame()
     m_ROIRightTxt->PopEventHandler(true);
     m_ROITopTxt->PopEventHandler(true);
     m_ROIBottomTxt->PopEventHandler(true);
+    for (int i=0; i < m_ToggleButtons.size(); i++)
+    {
+        m_ToggleButtons[i]->PopEventHandler();
+    }
     m_pano.removeObserver(this);
     DEBUG_TRACE("dtor end");
 }
@@ -611,7 +632,7 @@ void GLPreviewFrame::panoramaChanged(Panorama &pano)
         assert((int) params.size() == nParam);
         for (int i=0; i < nParam; i++) {
             wxString val = wxString(doubleToString(params[i],1).c_str(), wxConvLocal);
-            m_projParamTextCtrl[i]->SetValue(wxString(val.c_str(), wxConvLocal));
+            m_projParamTextCtrl[i]->SetValue(wxString(val.wc_str(), wxConvLocal));
             m_projParamSlider[i]->SetValue(utils::roundi(params[i]));
         }
     }
@@ -638,6 +659,11 @@ void GLPreviewFrame::panoramaChanged(Panorama &pano)
     m_ROIRightTxt->SetValue(wxString::Format(wxT("%d"), opts.getROI().right() ));
     m_ROITopTxt->SetValue(wxString::Format(wxT("%d"), opts.getROI().top() ));
     m_ROIBottomTxt->SetValue(wxString::Format(wxT("%d"), opts.getROI().bottom() ));
+    
+    if(m_showProjectionHints)
+    {
+        ShowProjectionWarnings();
+    };
 }
 
 void GLPreviewFrame::panoramaImagesChanged(Panorama &pano, const UIntSet &changed)
@@ -655,6 +681,7 @@ void GLPreviewFrame::panoramaImagesChanged(Panorama &pano, const UIntSet &change
     for (int i=nrButtons-1; i>=(int)nrImages; i--)
     {
         m_ButtonSizer->Detach(m_ToggleButtonPanel[i]);
+        m_ToggleButtons[i]->PopEventHandler();
         delete m_ToggleButtons[i];
         delete m_ToggleButtonPanel[i];
         m_ToggleButtons.pop_back();
@@ -741,6 +768,11 @@ void GLPreviewFrame::panoramaImagesChanged(Panorama &pano, const UIntSet &change
     };
 }
 
+void GLPreviewFrame::redrawPreview()
+{
+    m_GLViewer->Redraw();
+}
+
 
 void GLPreviewFrame::OnClose(wxCloseEvent& event)
 {
@@ -856,33 +888,68 @@ void GLPreviewFrame::OnNumTransform(wxCommandEvent & e)
 {
     if (m_pano.getNrOfImages() == 0) return;
 
-    wxString text = XRCCTRL(*this,"input_yaw",wxTextCtrl)->GetValue();
+    wxString text;
     double y;
-    if(!utils::stringToDouble(std::string(text.mb_str(wxConvLocal)), y))
-    {
-        wxBell();
-        wxMessageBox(_("Yaw value must be numeric."),_("Warning"),wxOK | wxICON_ERROR,this);
-        return;
-    }
-    text = XRCCTRL(*this,"input_pitch",wxTextCtrl)->GetValue();
     double p;
-    if(!utils::stringToDouble(std::string(text.mb_str(wxConvLocal)), p)) 
-    {
-        wxBell();
-        wxMessageBox(_("Pitch value must be numeric."),_("Warning"),wxOK | wxICON_ERROR,this);
-        return;
-    }
-    text = XRCCTRL(*this,"input_roll",wxTextCtrl)->GetValue();
     double r;
-    if(!utils::stringToDouble(std::string(text.mb_str(wxConvLocal)), r)) 
-    {
-        wxBell();
-        wxMessageBox(_("Roll value must be numeric."),_("Warning"),wxOK | wxICON_ERROR,this);
-        return;
+    double x;
+    double z;
+
+    int index = m_DragModeChoice->GetSelection();
+    switch (index) {
+        case 0: //normal
+            text = XRCCTRL(*this,"input_yaw",wxTextCtrl)->GetValue();
+            if(!utils::stringToDouble(std::string(text.mb_str(wxConvLocal)), y))
+            {
+                wxBell();
+                wxMessageBox(_("Yaw value must be numeric."),_("Warning"),wxOK | wxICON_ERROR,this);
+                return;
+            }
+            text = XRCCTRL(*this,"input_pitch",wxTextCtrl)->GetValue();
+            if(!utils::stringToDouble(std::string(text.mb_str(wxConvLocal)), p))
+            {
+                wxBell();
+                wxMessageBox(_("Pitch value must be numeric."),_("Warning"),wxOK | wxICON_ERROR,this);
+                return;
+            }
+            text = XRCCTRL(*this,"input_roll",wxTextCtrl)->GetValue();
+            if(!utils::stringToDouble(std::string(text.mb_str(wxConvLocal)), r))
+            {
+                wxBell();
+                wxMessageBox(_("Roll value must be numeric."),_("Warning"),wxOK | wxICON_ERROR,this);
+                return;
+            }
+            GlobalCmdHist::getInstance().addCommand(
+                    new PT::RotatePanoCmd(m_pano, y, p, r)
+                );
+            break;
+        case 1: //mosaic
+            text = XRCCTRL(*this,"input_x",wxTextCtrl)->GetValue();
+            if(!utils::stringToDouble(std::string(text.mb_str(wxConvLocal)), x))
+            {
+                wxBell();
+                wxMessageBox(_("X value must be numeric."),_("Warning"),wxOK | wxICON_ERROR,this);
+                return;
+            }
+            text = XRCCTRL(*this,"input_y",wxTextCtrl)->GetValue();
+            if(!utils::stringToDouble(std::string(text.mb_str(wxConvLocal)), y))
+            {
+                wxBell();
+                wxMessageBox(_("Y value must be numeric."),_("Warning"),wxOK | wxICON_ERROR,this);
+                return;
+            }
+            text = XRCCTRL(*this,"input_z",wxTextCtrl)->GetValue();
+            if(!utils::stringToDouble(std::string(text.mb_str(wxConvLocal)), z))
+            {
+                wxBell();
+                wxMessageBox(_("Z value must be numeric."),_("Warning"),wxOK | wxICON_ERROR,this);
+                return;
+            }
+            GlobalCmdHist::getInstance().addCommand(
+                    new PT::TranslatePanoCmd(m_pano, x, y, z)
+                );
+            break;
     }
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::RotatePanoCmd(m_pano, y, p, r)
-        );
 }
 
 void GLPreviewFrame::OnExposureChanged(wxCommandEvent & e)
@@ -1017,24 +1084,19 @@ void GLPreviewFrame::OnDragChoice(wxCommandEvent & e)
 {
     if (e.GetEventObject() == m_DragModeChoice)
     {
-        if (drag_tool) {
-        
-		    int index = m_DragModeChoice->GetSelection();
-		    switch (index) {
-		    	case 0: //normal
-		    		drag_tool->setDragMode(PreviewDragTool::drag_mode_normal);
-		    	break; 
-		    	case 1: //mosaic
-		    		drag_tool->setDragMode(PreviewDragTool::drag_mode_mosaic);
-		    	break;
-		    }
-            XRCCTRL(*this,"label_yaw",wxStaticText)->Enable(index==0);
-            XRCCTRL(*this,"input_yaw",wxTextCtrl)->Enable(index==0);
-            XRCCTRL(*this,"label_pitch",wxStaticText)->Enable(index==0);
-            XRCCTRL(*this,"input_pitch",wxTextCtrl)->Enable(index==0);
-            XRCCTRL(*this,"label_roll",wxStaticText)->Enable(index==0);
-            XRCCTRL(*this,"input_roll",wxTextCtrl)->Enable(index==0);
-            XRCCTRL(*this,"apply_num_transform",wxButton)->Enable(index==0);
+        if (drag_tool)
+        {
+            int index = m_DragModeChoice->GetSelection();
+            switch (index) {
+                case 0: //normal
+                    drag_tool->setDragMode(PreviewDragTool::drag_mode_normal);
+                    break;
+                case 1: //mosaic
+                    drag_tool->setDragMode(PreviewDragTool::drag_mode_mosaic);
+                    break;
+            }
+            // adjust the layout
+            GLPreviewFrame::DragChoiceLayout(index);
         }
     }
     else
@@ -1043,6 +1105,39 @@ void GLPreviewFrame::OnDragChoice(wxCommandEvent & e)
     }
 }
 
+void GLPreviewFrame::DragChoiceLayout( int index )
+{
+            // visibility of controls based on selected drag mode
+            XRCCTRL(*this,"label_yaw",wxStaticText)->Show(index==0);
+            XRCCTRL(*this,"input_yaw",wxTextCtrl)->Show(index==0);
+            XRCCTRL(*this,"label_pitch",wxStaticText)->Show(index==0);
+            XRCCTRL(*this,"input_pitch",wxTextCtrl)->Show(index==0);
+            XRCCTRL(*this,"label_roll",wxStaticText)->Show(index==0);
+            XRCCTRL(*this,"input_roll",wxTextCtrl)->Show(index==0);
+            XRCCTRL(*this,"label_x",wxStaticText)->Show(index==1);
+            XRCCTRL(*this,"input_x",wxTextCtrl)->Show(index==1);
+            XRCCTRL(*this,"label_y",wxStaticText)->Show(index==1);
+            XRCCTRL(*this,"input_y",wxTextCtrl)->Show(index==1);
+            XRCCTRL(*this,"label_z",wxStaticText)->Show(index==1);
+            XRCCTRL(*this,"input_z",wxTextCtrl)->Show(index==1);
+            XRCCTRL(*this,"apply_num_transform",wxButton)->Show(1);
+            // redraw layout to compress empty space
+            XRCCTRL(*this,"label_yaw",wxStaticText)->GetContainingSizer()->Layout();
+            XRCCTRL(*this,"input_yaw",wxTextCtrl)->GetContainingSizer()->Layout();
+            XRCCTRL(*this,"label_pitch",wxStaticText)->GetContainingSizer()->Layout();
+            XRCCTRL(*this,"input_pitch",wxTextCtrl)->GetContainingSizer()->Layout();
+            XRCCTRL(*this,"label_roll",wxStaticText)->GetContainingSizer()->Layout();
+            XRCCTRL(*this,"input_roll",wxTextCtrl)->GetContainingSizer()->Layout();
+            XRCCTRL(*this,"label_x",wxStaticText)->GetContainingSizer()->Layout();
+            XRCCTRL(*this,"input_x",wxTextCtrl)->GetContainingSizer()->Layout();
+            XRCCTRL(*this,"label_y",wxStaticText)->GetContainingSizer()->Layout();
+            XRCCTRL(*this,"input_y",wxTextCtrl)->GetContainingSizer()->Layout();
+            XRCCTRL(*this,"label_z",wxStaticText)->GetContainingSizer()->Layout();
+            XRCCTRL(*this,"input_z",wxTextCtrl)->GetContainingSizer()->Layout();
+            XRCCTRL(*this,"apply_num_transform",wxButton)->GetContainingSizer()->Layout();
+}
+
+
 void GLPreviewFrame::OnDefaultExposure( wxCommandEvent & e )
 {
     if (m_pano.getNrOfImages() > 0) {
@@ -1423,13 +1518,13 @@ void GLPreviewFrame::OnFullScreen(wxCommandEvent & e)
 void GLPreviewFrame::OnUndo(wxCommandEvent &e)
 {
     wxCommandEvent dummy(wxEVT_COMMAND_MENU_SELECTED, XRCID("ID_EDITUNDO"));
-    m_parent->AddPendingEvent(dummy);
+    m_parent->GetEventHandler()->AddPendingEvent(dummy);
 };
 
 void GLPreviewFrame::OnRedo(wxCommandEvent &e)
 {
     wxCommandEvent dummy(wxEVT_COMMAND_MENU_SELECTED, XRCID("ID_EDITREDO"));
-    m_parent->AddPendingEvent(dummy);
+    m_parent->GetEventHandler()->AddPendingEvent(dummy);
 };
 
 void GLPreviewFrame::SetMode(int newMode)
@@ -1625,3 +1720,108 @@ void GLPreviewFrame::OnLayoutScaleChange(wxScrollEvent &e)
         m_GLViewer->Refresh();
     };
 };
+
+void GLPreviewFrame::ShowProjectionWarnings()
+{
+    PanoramaOptions opts = m_pano.getOptions();
+    double hfov = opts.getHFOV();
+    double vfov = opts.getVFOV();
+    double maxfov = hfov > vfov ? hfov : vfov;
+    wxString message;
+    // If this is set to true, offer rectilinear as an alternative if it fits.
+    bool rectilinear_option = false;
+    switch (opts.getProjection()) {
+        case HuginBase::PanoramaOptions::RECTILINEAR:
+            if (maxfov > 120.0) {
+                            // wide rectilinear image
+                message = _("With a wide field of view, panoramas with rectilinear projection get very stretched towards the edges.\n");
+                if (vfov < 110) {
+                    message += _("Since the field of view is only very wide in the horizontal direction, try a cylindrical projection instead.");
+                } else {
+                    message += _("For a very wide panorama, try equirectangular projection instead.");
+                }
+                message += _(" You could also try Panini projection.");
+            }
+            break;
+        case HuginBase::PanoramaOptions::CYLINDRICAL:
+            if (vfov > 120.0) {
+                message = _("With a wide vertical field of view, panoramas with cylindrical projection get very stretched at the top and bottom.\nAn equirectangular projection would fit the same content in less vertical space.");
+            } else rectilinear_option = true;
+            break;
+        case HuginBase::PanoramaOptions::EQUIRECTANGULAR:
+            if (vfov < 110.0 && hfov > 120.0)
+            {
+                message = _("Since the vertical field of view is not too wide, you could try setting the panorama projection to cylindrical.\nCylindrical projection preserves vertical lines, unlike equirectangular.");
+            } else rectilinear_option = true;
+            break;
+        case HuginBase::PanoramaOptions::FULL_FRAME_FISHEYE:
+            if (maxfov < 280.0) {
+                rectilinear_option = true;
+                message = _("Stereographic projection is conformal, unlike this Fisheye panorama projection.\nA conformal projection preserves angles around a point, which often makes it easier on the eye.");
+            }
+            break;
+        case HuginBase::PanoramaOptions::STEREOGRAPHIC:
+            if (maxfov > 300.0) {
+                message = _("Panoramas with stereographic projection and a very wide field of view stretch the image around the edges a lot.\nThe Fisheye panorama projection compresses it, so you can fit in a wide field of view and still have a reasonable coverage of the middle.");
+            } else rectilinear_option = true;
+            break;
+        default:
+            rectilinear_option = true;
+    }
+    if (rectilinear_option && maxfov < 110.0) {
+        message = _("Setting the panorama to rectilinear projection would keep the straight lines straight.");
+    }
+    if (message.IsEmpty()) {
+        // no message needed.
+#if wxCHECK_VERSION(2, 9, 1)
+        m_infoBar->Dismiss();
+#else
+        if (m_projectionStatusPushed) {
+            m_projectionStatusPushed = false;
+            GetStatusBar()->PopStatusText();
+        }
+#endif
+    } else {
+#if wxCHECK_VERSION(2, 9, 1)
+        m_infoBar->ShowMessage(message, wxICON_INFORMATION);
+#else
+        if (m_projectionStatusPushed) {
+            GetStatusBar()->PopStatusText();
+        }
+        /** @todo The message doesn't really fit in the status bar, so we should have some other GUI arrangement or remove this feature.
+         * On my system, the status bar remains too short to contain the two
+         * lines of text in the message.
+         */
+        GetStatusBar()->PushStatusText(message);
+        m_projectionStatusPushed = true;
+#endif
+    }
+};
+
+void GLPreviewFrame::SetShowProjectionHints(bool new_value)
+{
+    m_showProjectionHints=new_value;
+#if wxCHECK_VERSION(2,9,1)
+    if(!m_showProjectionHints)
+    {
+        m_infoBar->Dismiss();
+    };
+#endif
+};
+
+void GLPreviewFrame::OnHideProjectionHints(wxCommandEvent &e)
+{
+    wxMessageBox(_("You have hidden the infobar, which shows hints about selection of projection.\nIf you want to see the bar again, activate the bar in the preferences again."),
+#ifdef __WXMSW__
+        _("Hugin"),
+#else
+        wxT(""),
+#endif
+        wxOK | wxICON_INFORMATION, this);
+
+    wxConfigBase* cfg=wxConfigBase::Get();
+    cfg->Write(wxT("/GLPreviewWindow/ShowProjectionHints"), false);
+    m_showProjectionHints=false;
+    cfg->Flush();
+    e.Skip();
+};
