@@ -49,6 +49,8 @@ public:
     static void RefreshWrapper(void *obj);
     void SetUpContext();
     void SetPhotometricCorrect(bool state);
+    void SetLayoutMode(bool state);
+    void SetLayoutScale(double scale);
     
     ViewState * m_view_state;
 protected:
@@ -69,7 +71,7 @@ protected:
     wxGLContext *m_glContext;
     PT::Panorama  * m_pano;
     
-    bool started_creation, redrawing;
+    bool started_creation, initialised_glew, redrawing;
     vigra::Diff2D offset;
     GLPreviewFrame *frame;
 };
