@@ -52,9 +52,9 @@ HDRMergeOptionsDialog::HDRMergeOptionsDialog(wxWindow *parent)
     wxXmlResource::Get()->LoadDialog(this, parent, wxT("hdrmerge_options_dialog"));
 
 #ifdef __WXMSW__
-    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/icon.ico"),wxBITMAP_TYPE_ICO);
+    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/hugin.ico"),wxBITMAP_TYPE_ICO);
 #else
-    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/icon.png"),wxBITMAP_TYPE_PNG);
+    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/hugin.png"),wxBITMAP_TYPE_PNG);
 #endif
     SetIcon(myIcon);
     m_mode=XRCCTRL(*this,"hdrmerge_option_mode",wxChoice);
@@ -80,11 +80,21 @@ void HDRMergeOptionsDialog::SetCommandLineArgument(wxString cmd)
 	// parse arguments
     static const wxCmdLineEntryDesc cmdLineDesc[] =
     {
-        { wxCMD_LINE_OPTION, wxT("m"), NULL, NULL, wxCMD_LINE_VAL_STRING },     
+    // wxT is required previous to wxWidgets 2.9, and not recommended after it.
+    // Here it causes errors converting between const wchar_t * and const char*.
+#if wxCHECK_VERSION(2,9,0)
+        { wxCMD_LINE_OPTION, "m", NULL, NULL, wxCMD_LINE_VAL_STRING },
+        { wxCMD_LINE_SWITCH, "c", NULL, NULL},
+        { wxCMD_LINE_OPTION, "i", NULL, NULL, wxCMD_LINE_VAL_NUMBER },
+        { wxCMD_LINE_OPTION, "s", NULL, NULL, wxCMD_LINE_VAL_STRING },
+        { wxCMD_LINE_OPTION, "a", NULL, NULL, wxCMD_LINE_VAL_STRING },
+#else
+        { wxCMD_LINE_OPTION, wxT("m"), NULL, NULL, wxCMD_LINE_VAL_STRING },
         { wxCMD_LINE_SWITCH, wxT("c"), NULL, NULL},
         { wxCMD_LINE_OPTION, wxT("i"), NULL, NULL, wxCMD_LINE_VAL_NUMBER },
         { wxCMD_LINE_OPTION, wxT("s"), NULL, NULL, wxCMD_LINE_VAL_STRING },
         { wxCMD_LINE_OPTION, wxT("a"), NULL, NULL, wxCMD_LINE_VAL_STRING },
+#endif
         { wxCMD_LINE_NONE }
     };
     wxCmdLineParser parser;
