@@ -40,12 +40,11 @@
 #include "hugin/ImagesPanel.h"
 #include "hugin/CommandHistory.h"
 #include "hugin/TextKillFocusHandler.h"
-#include "base_wx/ImageCache.h"
 #include "hugin/CPEditorPanel.h"
 #include "hugin/ImagesList.h"
 #include "hugin/MainFrame.h"
 #include "hugin/huginApp.h"
-#include "hugin/AutoCtrlPointCreator.h"
+#include "icpfind/AutoCtrlPointCreator.h"
 #include "hugin/config_defaults.h"
 #include "base_wx/MyProgressDialog.h"
 #include "base_wx/PTWXDlg.h"
@@ -171,9 +170,10 @@ bool ImagesPanel::Create(wxWindow *parent, wxWindowID id, const wxPoint& pos, co
     XRCCTRL(*this, "images_text_Y", wxTextCtrl)->PushEventHandler(new TextKillFocusHandler(this));
     XRCCTRL(*this, "images_text_Z", wxTextCtrl)->PushEventHandler(new TextKillFocusHandler(this));
 
-    m_empty.LoadFile(huginApp::Get()->GetXRCPath() +
-                     wxT("data/") + wxT("druid.images.128.png"),
-                     wxBITMAP_TYPE_PNG);
+    m_empty.Create(0,0);
+//    m_empty.LoadFile(huginApp::Get()->GetXRCPath() +
+//                     wxT("data/") + wxT("transparent.png"),
+//                     wxBITMAP_TYPE_PNG);
     m_smallImgCtrl->SetBitmap(m_empty);
 
     wxListEvent ev;
@@ -185,7 +185,7 @@ bool ImagesPanel::Create(wxWindow *parent, wxWindowID id, const wxPoint& pos, co
     wxConfigBase* config=wxConfigBase::Get();
     m_degDigits = config->Read(wxT("/General/DegreeFractionalDigitsEdit"),3);
     //read autopano generator settings
-    cpdetector_config.Read(config);
+    cpdetector_config.Read(config,huginApp::Get()->GetDataPath()+wxT("default.setting"));
     //write current autopano generator settings
     cpdetector_config.Write(config);
     config->Flush();
@@ -326,7 +326,7 @@ void ImagesPanel::SIFTMatching(wxCommandEvent & e)
     CPVector cps = matcher.automatch(cpdetector_config.settings[m_CPDetectorChoice->GetSelection()],
         *pano, selImg, nFeatures,this);
     wxString msg;
-    wxMessageBox(wxString::Format(_("Added %d control points"), cps.size()), _("Control point detector result"),wxOK|wxICON_INFORMATION,this);
+    wxMessageBox(wxString::Format(_("Added %lu control points"), (unsigned long) cps.size()), _("Control point detector result"),wxOK|wxICON_INFORMATION,this);
     GlobalCmdHist::getInstance().addCommand(
             new PT::AddCtrlPointsCmd(*pano, cps)
                                            );
@@ -376,7 +376,7 @@ void ImagesPanel::OnCleanCP(wxCommandEvent & e)
                         );
         NrRemoved+=CPremove.size();
     };
-    wxMessageBox(wxString::Format(_("Removed %d control points"), NrRemoved), _("Cleaning"),wxOK|wxICON_INFORMATION,this);
+    wxMessageBox(wxString::Format(_("Removed %u control points"), NrRemoved), _("Cleaning"),wxOK|wxICON_INFORMATION,this);
 };
 
 void ImagesPanel::OnVarTextChanged ( wxCommandEvent & e )
@@ -500,7 +500,6 @@ void ImagesPanel::ListSelectionChanged(wxListEvent & e)
             // multiselection, clear all values
             // we don't know which images parameters to show.
             ClearImgParameters();
-            ClearImgExifInfo();
             m_optAnchorButton->Disable();
             m_colorAnchorButton->Disable();
             m_moveDownButton->Disable();
@@ -697,28 +696,46 @@ void ImagesPanel::UpdatePreviewImage()
     if (m_showImgNr < 0 || m_showImgNr >= pano->getNrOfImages()) {
         return;
     }
-    ImageCache::EntryPtr cacheEntry = ImageCache::getInstance().getSmallImage(
+    ImageCache::EntryPtr cacheEntry = ImageCache::getInstance().getSmallImageIfAvailable(
             pano->getImage(m_showImgNr).getFilename());
-    wxImage img = imageCacheEntry2wxImage(cacheEntry); 
-
-    double iRatio = img.GetWidth() / (double) img.GetHeight();
-
-    wxSize sz;
-    // estimate image size
-    
-    sz = m_smallImgCtrl->GetContainingSizer()->GetSize();
-    double sRatio = (double)sz.GetWidth() / sz.GetHeight();
-    if (iRatio > sRatio) {
-        // image is wider than screen, display landscape
-        sz.SetHeight((int) (sz.GetWidth() / iRatio));
+    if (!cacheEntry.get())
+    {
+        // image currently isn't loaded.
+        // Instead of loading and displaying the image now, request it for
+        // later. Then the user can switch between images in the list quickly,
+        // even when not all images previews are in the cache.
+        thumbnail_request = ImageCache::getInstance().requestAsyncSmallImage(
+                pano->getImage(m_showImgNr).getFilename());
+        // When the image is ready, try this function again.
+        thumbnail_request->ready.connect(
+            boost::bind(&ImagesPanel::UpdatePreviewImage, this));
     } else {
-        // portrait
-        sz.SetWidth((int) (sz.GetHeight() * iRatio));
+        // forget any request now the image has loaded.
+        thumbnail_request = ImageCache::RequestPtr();
+        wxImage img = imageCacheEntry2wxImage(cacheEntry); 
+
+        double iRatio = img.GetWidth() / (double) img.GetHeight();
+
+        wxSize sz;
+        // estimate image size
+        
+        sz = m_smallImgCtrl->GetContainingSizer()->GetSize();
+        double sRatio = (double)sz.GetWidth() / sz.GetHeight();
+        if (iRatio > sRatio) {
+            // image is wider than screen, display landscape
+            sz.SetHeight((int) (sz.GetWidth() / iRatio));
+        } else {
+            // portrait
+            sz.SetWidth((int) (sz.GetHeight() * iRatio));
+        }
+        // Make sure the size is positive:
+        // on a small window, m_smallImgCtrl can have 0 width.
+        sz.IncTo(wxSize(1,1));
+        wxImage scaled = img.Scale(sz.GetWidth(),sz.GetHeight());
+        m_smallImgCtrl->SetBitmap(wxBitmap(scaled));
+        m_smallImgCtrl->GetParent()->Layout();
+        m_smallImgCtrl->Refresh();
     }
-    wxImage scaled = img.Scale(sz.GetWidth(),sz.GetHeight());
-    m_smallImgCtrl->SetBitmap(wxBitmap(scaled));
-    m_smallImgCtrl->GetParent()->Layout();
-    m_smallImgCtrl->Refresh();
 }
 
 
@@ -861,8 +878,8 @@ void ImagesPanel::OnRemoveCtrlPoints(wxCommandEvent & e)
             cpsToDelete.insert(it - cps.begin());
         }
     }
-    int r =wxMessageBox(wxString::Format(_("Really Delete %d control points?"),
-                                         cpsToDelete.size()),
+    int r =wxMessageBox(wxString::Format(_("Really Delete %lu control points?"),
+                                         (unsigned long int) cpsToDelete.size()),
                         _("Delete Control Points"),
                         wxICON_QUESTION | wxYES_NO);
     if (r == wxYES) {
@@ -878,6 +895,7 @@ void ImagesPanel::OnMoveImageDown(wxCommandEvent & e)
         unsigned int i1 = *selImg.begin();
         unsigned int i2 = i1+1;
         if (i2 < pano->getNrOfImages() ) {
+            images_list->SetItemState(i1,0,wxLIST_STATE_SELECTED);
             GlobalCmdHist::getInstance().addCommand(
                 new SwapImagesCmd(*pano,i1, i2)
             );
@@ -897,6 +915,7 @@ void ImagesPanel::OnMoveImageUp(wxCommandEvent & e)
         unsigned int i1 = *selImg.begin();
         unsigned int i2 = i1 -1;
         if (i1 > 0) {
+            images_list->SetItemState(i1,0,wxLIST_STATE_SELECTED);
             GlobalCmdHist::getInstance().addCommand(
                 new SwapImagesCmd(*pano,i1, i2)
             );
@@ -994,7 +1013,6 @@ IMPLEMENT_DYNAMIC_CLASS(ImagesPanelXmlHandler, wxXmlResourceHandler)
 void ImagesPanel::OnCelesteButton(wxCommandEvent & e)
 {
     const UIntSet & selImg = images_list->GetSelected();
-    unsigned int total_removed = 0;
 
     if ( selImg.size() == 0)
     {
@@ -1002,119 +1020,74 @@ void ImagesPanel::OnCelesteButton(wxCommandEvent & e)
     }
     else
     {	
-        ProgressReporterDialog progress(selImg.size()+1, _("Running Celeste"), _("Running Celeste"),this);
+        ProgressReporterDialog progress(selImg.size()+2, _("Running Celeste"), _("Running Celeste"),this);
+        MainFrame::Get()->SetStatusText(_("searching for cloud-like control points..."),0);
+        progress.increaseProgress(1.0, std::wstring(wxString(_("Loading model file")).wc_str(wxConvLocal)));
 
+        struct celeste::svm_model* model=MainFrame::Get()->GetSVMModel();
+        if(model==NULL)
+        {
+            MainFrame::Get()->SetStatusText(wxT(""),0);
+            return;
+        };
+
+        // Get Celeste parameters
+        wxConfigBase *cfg = wxConfigBase::Get();
+        // SVM threshold
+        double threshold = HUGIN_CELESTE_THRESHOLD;
+        cfg->Read(wxT("/Celeste/Threshold"), &threshold, HUGIN_CELESTE_THRESHOLD);
+
+        // Mask resolution - 1 sets it to fine
+        bool t = (cfg->Read(wxT("/Celeste/Filter"), HUGIN_CELESTE_FILTER) == 0);
+        int radius=(t)?10:20;
         DEBUG_TRACE("Running Celeste");
 
-        // determine file name of SVM model file
-        // get XRC path from application
-        wxString wxstrModelFileName = huginApp::Get()->GetDataPath() + wxT(HUGIN_CELESTE_MODEL);
-        // convert wxString to string
-        string strModelFileName(wxstrModelFileName.mb_str(wxConvUTF8));
-		
-        // SVM model file
-        if (! wxFile::Exists(wxstrModelFileName) ) {
-            wxMessageBox(_("Celeste model expected in ") + wxstrModelFileName +_(" not found, Hugin needs to be properly installed." ), _("Fatal Error"));
-            return ;
-        }
-
-        for (UIntSet::const_iterator itr = selImg.begin(); itr != selImg.end(); ++itr) {
-
-            progress.increaseProgress(1.0, std::wstring(wxString(_("Running Celeste")).wc_str(wxConvLocal)));
-
-            const CPVector & controlPoints = pano->getCtrlPoints();
-            unsigned int removed = 0;
-            const unsigned int imgNr = *itr;
-
-            gNumLocs = 0;
-            for (PT::CPVector::const_iterator it = controlPoints.begin(); it != controlPoints.end(); ++it) {
-                PT::ControlPoint point = *it;
-                if (imgNr == point.image1Nr){
-                    gNumLocs++;				
-                }
-                if (imgNr == point.image2Nr){
-                    gNumLocs++;				
-                }
-            }		
-
-            // Create the storage matrix
-            gLocations = CreateMatrix( (int)0, gNumLocs, 2);
-            unsigned int glocation_counter = 0;
-            unsigned int cp_counter = 0;	
-            vector<unsigned int> global_cp_nr;
-
-            for (PT::CPVector::const_iterator it = controlPoints.begin(); it != controlPoints.end(); ++it) {
-                PT::ControlPoint point = *it;
-
-                if (imgNr == point.image1Nr){
-                    //cout << "---imgNr = " << imgNr << " point.image1Nr = " << point.image1Nr << endl;	
-                    gLocations[glocation_counter][0] = (int)point.x1;
-                    gLocations[glocation_counter][1] = (int)point.y1;
-                    global_cp_nr.push_back(cp_counter);	
-                    glocation_counter++;				
-                }
-                if (imgNr == point.image2Nr){
-                    //cout << "---imgNr = " << imgNr << " point.image1Nr = " << point.image1Nr << endl;	
-                    gLocations[glocation_counter][0] = (int)point.x2;
-                    gLocations[glocation_counter][1] = (int)point.y2;
-                    global_cp_nr.push_back(cp_counter);	
-                    glocation_counter++;				
-                }
-                cp_counter++;	
-            }
-
-            // SVM threshold
-            double threshold = HUGIN_CELESTE_THRESHOLD;
-            wxConfigBase::Get()->Read(wxT("/Celeste/Threshold"), &threshold, HUGIN_CELESTE_THRESHOLD);
-
-            // Mask resolution - 1 sets it to fine
-            bool t = (wxConfigBase::Get()->Read(wxT("/Celeste/Filter"), HUGIN_CELESTE_FILTER) != 0);
-            if (t){
-                //cerr <<"---Celeste--- Using small filter" << endl;
-                gRadius = 10;
-                spacing = (gRadius * 2) + 1;
-            }
-
+        UIntSet cpsToRemove;
+        for (UIntSet::const_iterator it=selImg.begin(); it!=selImg.end(); it++)
+        {
             // Image to analyse
-            string imagefile = pano->getImage(*itr).getFilename();
-
-            // Print progress
-            MainFrame::Get()->SetStatusText(_("searching for cloud-like control points..."),0);
-
-            // Vector to store Gabor filter responses
-            vector<double> svm_responses_im;
-            string mask_format = "PNG";
-            unsigned int mask = 0;
-
-            // Get responses
-            get_gabor_response(imagefile, mask, strModelFileName, threshold, mask_format, svm_responses_im);
-
-            MainFrame::Get()->SetStatusText(_("classifying control points..."),0);
-
-            UIntSet cpToRemove;
-            // Print SVM results
-            for (unsigned int c = 0; c < svm_responses_im.size(); c++){
-                if (svm_responses_im[c] >= threshold){
-                    cpToRemove.insert(global_cp_nr[c]);
-                    removed++;
-                    total_removed++;
-                    cout << "CP: " << c << "\tSVM Score: " << svm_responses_im[c] << "\tremoved." << endl;
-                }
-            }
-            if (removed)
+            HuginBase::CPointVector cps=pano->getCtrlPointsVectorForImage(*it);
+            if(cps.size()==0)
             {
-                cout << endl;
+                progress.increaseProgress(1.0, std::wstring(wxString(_("Running Celeste")).wc_str(wxConvLocal)));
+                continue;
+            };
+            ImageCache::EntryPtr img=ImageCache::getInstance().getImage(pano->getImage(*it).getFilename());
+            vigra::UInt16RGBImage in;
+            if(img->image16->width()>0)
+            {
+                in.resize(img->image16->size());
+                vigra::copyImage(srcImageRange(*(img->image16)),destImage(in));
             }
-            if(cpToRemove.size()>0)
-                GlobalCmdHist::getInstance().addCommand(
-                    new PT::RemoveCtrlPointsCmd(*pano,cpToRemove)
-                    );
+            else
+            {
+                ImageCache::ImageCacheRGB8Ptr im8=img->get8BitImage();
+                in.resize(im8->size());
+                vigra::transformImage(srcImageRange(*im8),destImage(in),vigra::functor::Arg1()*vigra::functor::Param(65535/255));
+            };
+            UIntSet cloudCP=celeste::getCelesteControlPoints(model,in,cps,radius,threshold,800);
+            in.resize(0,0);
+            if(cloudCP.size()>0)
+            {
+                for(UIntSet::const_iterator it2=cloudCP.begin();it2!=cloudCP.end();it2++)
+                {
+                    cpsToRemove.insert(*it2);
+                };
+            };
+            progress.increaseProgress(1.0, std::wstring(wxString(_("Running Celeste")).wc_str(wxConvLocal)));
+        };
 
-        }
+        if(cpsToRemove.size()>0)
+        {
+            GlobalCmdHist::getInstance().addCommand(
+                new PT::RemoveCtrlPointsCmd(*pano,cpsToRemove)
+                );
+        };
+
+        progress.increaseProgress(1.0, std::wstring(wxString(_("Running Celeste")).wc_str(wxConvLocal)));
+        wxMessageBox(wxString::Format(_("Removed %lu control points"), (unsigned long int) cpsToRemove.size()), _("Celeste result"),wxOK|wxICON_INFORMATION,this);
+        DEBUG_TRACE("Finished running Celeste");
         MainFrame::Get()->SetStatusText(wxT(""),0);
-
     }
-
-    wxMessageBox(wxString::Format(_("Removed %d control points"), total_removed), _("Celeste result"),wxOK|wxICON_INFORMATION,this);
 }
 
