@@ -3,7 +3,7 @@
  *
  *  @author Kai-Uwe Behrmann <web@tiscali.de>
  *
- *  $Id: LensPanel.h 3694 2009-03-01 22:47:11Z brunopostle $
+ *  $Id$
  *
  *  This is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -27,6 +27,8 @@
 #include "PT/Panorama.h"
 #include "hugin/MainFrame.h"
 
+#include <panodata/StandardImageVariableGroups.h>
+
 class ImagesListLens;
 
 /** Define the second the Lens panel
@@ -78,6 +80,9 @@ class LensPanel: public wxPanel, public PT::PanoramaObserver
      */
     void UpdateLensDisplay ();
 
+    /** enabled or disable the controls depending on selected images */
+    void EnableInputs();
+
     void EditVigCorr (wxCommandEvent & e);
 
  private:
@@ -125,6 +130,8 @@ class LensPanel: public wxPanel, public PT::PanoramaObserver
 
     // the model
     Panorama * pano;
+    
+    HuginBase::StandardImageVariableGroups * variable_groups;
 
     ImagesListLens * images_list;
 
