@@ -34,6 +34,7 @@
 #include "hugin/huginApp.h"
 #include "hugin/config_defaults.h"
 #include "hugin/PreferencesDialog.h"
+#include "hugin/CPDetectorDialog.h"
 
 // validators are working different somehow...
 //#define MY_STR_VAL(id, filter) { XRCCTRL(*this, "prefs_" #id, wxTextCtrl)->SetValidator(wxTextValidator(filter, &id)); }
@@ -75,6 +76,9 @@ BEGIN_EVENT_TABLE(PreferencesDialog, wxDialog)
     EVT_BUTTON(XRCID("pref_cpdetector_movedown"), PreferencesDialog::OnCPDetectorMoveDown)
     EVT_BUTTON(XRCID("pref_cpdetector_default"), PreferencesDialog::OnCPDetectorDefault)
     EVT_LISTBOX_DCLICK(XRCID("pref_cpdetector_list"), PreferencesDialog::OnCPDetectorListDblClick)
+    EVT_BUTTON(XRCID("pref_cpdetector_load"), PreferencesDialog::OnCPDetectorLoad)
+    EVT_BUTTON(XRCID("pref_cpdetector_save"), PreferencesDialog::OnCPDetectorSave)
+    EVT_BUTTON(XRCID("pref_cpdetector_help"), PreferencesDialog::OnCPDetectorHelp)
 //    EVT_CLOSE(RunOptimizerFrame::OnClose)
 END_EVENT_TABLE()
 
@@ -88,9 +92,9 @@ PreferencesDialog::PreferencesDialog(wxWindow *parent)
     wxXmlResource::Get()->LoadDialog(this, parent, wxT("pref_dialog"));
 
 #ifdef __WXMSW__
-    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/icon.ico"),wxBITMAP_TYPE_ICO);
+    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/hugin.ico"),wxBITMAP_TYPE_ICO);
 #else
-    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/icon.png"),wxBITMAP_TYPE_PNG);
+    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/hugin.png"),wxBITMAP_TYPE_PNG);
 #endif
     SetIcon(myIcon);
 
@@ -190,6 +194,12 @@ PreferencesDialog::PreferencesDialog(wxWindow *parent)
     this->SetBackgroundColour(XRCCTRL(*this, "prefs_ft_RotationStartAngle", wxSpinCtrl)->GetBackgroundColour());
 #endif
 
+#if wxCHECK_VERSION(2,9,1)
+    wxCheckBox* show_hints=XRCCTRL(*this,"pref_show_projection_hints",wxCheckBox);
+    show_hints->Enable(true);
+    show_hints->Show(true);
+#endif
+
     GetSizer()->SetSizeHints(this);
 //    GetSizer()->Layout();
 
@@ -278,7 +288,7 @@ void PreferencesDialog::OnPTStitcherExe(wxCommandEvent & e)
 #else
 		     wxT(""),
 #endif
-                    wxOPEN, wxDefaultPosition);
+                    wxFD_OPEN, wxDefaultPosition);
     if (dlg.ShowModal() == wxID_OK) {
 	XRCCTRL(*this, "prefs_pt_PTStitcherEXE", wxTextCtrl)->SetValue(
 		dlg.GetPath());
@@ -296,7 +306,7 @@ void PreferencesDialog::OnEditorExe(wxCommandEvent & e)
 #else
              wxT("(*)|*"),
 #endif
-                    wxOPEN, wxDefaultPosition);
+                    wxFD_OPEN, wxDefaultPosition);
     if (dlg.ShowModal() == wxID_OK) {
         XRCCTRL(*this, "prefs_ass_editor", wxTextCtrl)->SetValue(
                 dlg.GetPath());
@@ -313,7 +323,7 @@ void PreferencesDialog::OnEnblendExe(wxCommandEvent & e)
 #else
 		     wxT("*"),
 #endif
-                    wxOPEN, wxDefaultPosition);
+                    wxFD_OPEN, wxDefaultPosition);
     if (dlg.ShowModal() == wxID_OK) {
 	XRCCTRL(*this, "prefs_enblend_EnblendExe", wxTextCtrl)->SetValue(
 		dlg.GetPath());
@@ -329,7 +339,7 @@ void PreferencesDialog::OnEnfuseExe(wxCommandEvent & e)
 #else
 		     wxT("*"),
 #endif
-                    wxOPEN, wxDefaultPosition);
+                    wxFD_OPEN, wxDefaultPosition);
     if (dlg.ShowModal() == wxID_OK) {
 	XRCCTRL(*this, "prefs_enblend_EnfuseExe", wxTextCtrl)->SetValue(
 		dlg.GetPath());
@@ -427,6 +437,16 @@ void PreferencesDialog::UpdateDisplayData(int panel)
             DEBUG_WARN("Unknown language configured");
         }
 
+        // project naming convention
+        t = cfg->Read(wxT("ProjectNamingConvention"), HUGIN_PROJECT_NAMING_CONVENTION) == 1;
+        MY_BOOL_VAL("prefs_project_naming_convention", t);
+
+        // smart undo
+        t = cfg->Read(wxT("smartUndo"), HUGIN_SMART_UNDO) == 1;
+        MY_BOOL_VAL("prefs_smart_undo", t);
+
+        t = cfg->Read(wxT("/GLPreviewWindow/ShowProjectionHints"), HUGIN_SHOW_PROJECTION_HINTS) == 1;
+        MY_BOOL_VAL("pref_show_projection_hints", t)
 
         // cursor setting
 //    mem = cfg->Read(wxT("/CPImageCtrl/CursorType"), HUGIN_CP_CURSOR);
@@ -602,7 +622,13 @@ void PreferencesDialog::OnRestoreDefaults(wxCommandEvent & e)
             if (cpucount < 1) cpucount = 1;
             cfg->Write(wxT("/Nona/NumberOfThreads"), cpucount);
             // locale
-            cfg->Write(wxT("language"), HUGIN_LANGUAGE);
+            cfg->Write(wxT("language"), int(HUGIN_LANGUAGE));
+            // project naming convention
+            cfg->Write(wxT("ProjectNamingConvention"), HUGIN_PROJECT_NAMING_CONVENTION);
+            // smart undo
+            cfg->Write(wxT("smartUndo"), HUGIN_SMART_UNDO);
+            // projection hints
+            cfg->Write(wxT("/GLPreviewWindow/ShowProjectionHints"), HUGIN_SHOW_PROJECTION_HINTS);
             // druid
             cfg->Write(wxT("/PreviewFrame/showDruid"), HUGIN_PREVIEW_SHOW_DRUID);
         }
@@ -637,7 +663,7 @@ void PreferencesDialog::OnRestoreDefaults(wxCommandEvent & e)
         if (noteb->GetSelection() == 3) {
             /////
             /// AUTOPANO
-            cpdetector_config_edit.ResetToDefault();
+            cpdetector_config_edit.ReadFromFile(huginApp::Get()->GetDataPath()+wxT("default.setting"));
             cpdetector_config_edit.Write(cfg);
         }
         if (noteb->GetSelection() == 4) {
@@ -732,7 +758,12 @@ void PreferencesDialog::UpdateConfigData()
     long templ =  * static_cast<long *>(tmplp);
     cfg->Write(wxT("language"), templ);
     DEBUG_INFO("Language Selection ID: " << templ);
-    
+    // project naming convention
+    cfg->Write(wxT("ProjectNamingConvention"), MY_G_BOOL_VAL("prefs_project_naming_convention"));
+    // smart undo
+    cfg->Write(wxT("smartUndo"), MY_G_BOOL_VAL("prefs_smart_undo"));
+    // show projections hints
+    cfg->Write(wxT("/GLPreviewWindow/ShowProjectionHints"), MY_G_BOOL_VAL("pref_show_projection_hints"));
     // cursor
     //    cfg->Write(wxT("/CPImageCtrl/CursorType"), MY_G_SPIN_VAL("prefs_cp_CursorType"));
     // tempdir
@@ -863,3 +894,44 @@ void PreferencesDialog::OnCPDetectorListDblClick(wxCommandEvent &e)
 {
     OnCPDetectorEdit(e);
 };
+
+void PreferencesDialog::OnCPDetectorLoad(wxCommandEvent &e)
+{
+    wxFileDialog dlg(this,_("Load control point detector settings"),
+        wxConfigBase::Get()->Read(wxT("/actualPath"),wxT("")), wxEmptyString,
+        _("Control point detector settings (*.setting)|*.setting"),wxFD_OPEN | wxFD_FILE_MUST_EXIST);
+    if (dlg.ShowModal() == wxID_OK)
+    {
+        wxConfig::Get()->Write(wxT("/actualPath"), dlg.GetDirectory());  // remember for later
+        wxString fn = dlg.GetPath();
+        cpdetector_config_edit.ReadFromFile(fn);
+        cpdetector_config_edit.Write();
+        UpdateDisplayData(4);
+    };
+};
+
+void PreferencesDialog::OnCPDetectorSave(wxCommandEvent &e)
+{
+    wxFileDialog dlg(this,_("Save control point detector settings"),
+        wxConfigBase::Get()->Read(wxT("/actualPath"),wxT("")), wxEmptyString,
+        _("Control point detector settings (*.setting)|*.setting"),wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
+    if (dlg.ShowModal() == wxID_OK)
+    {
+        wxConfig::Get()->Write(wxT("/actualPath"), dlg.GetDirectory());  // remember for later
+        wxString fn = dlg.GetPath();
+#ifndef __WXMSW__
+        //append extension if not given
+        //not necessary on Windows, the wxFileDialog appends it automatic
+        if(fn.Right(8)!=wxT(".setting"))
+        {
+            fn.Append(wxT(".setting"));
+        };
+#endif
+        cpdetector_config_edit.WriteToFile(fn);
+    };
+};
+
+void PreferencesDialog::OnCPDetectorHelp(wxCommandEvent &e)
+{
+    MainFrame::Get()->DisplayHelp(wxT("/Control_Point_Detector_Parameters.html"));
+};
