@@ -111,7 +111,7 @@ BEGIN_EVENT_TABLE(PreviewFrame, wxFrame)
     EVT_BUTTON(PROJ_PARAM_RESET_ID, PreviewFrame::OnProjParameterReset)
 END_EVENT_TABLE()
 
-#define PF_STYLE (wxMAXIMIZE_BOX | wxRESIZE_BORDER | wxSYSTEM_MENU | wxCAPTION | wxCLOSE_BOX | wxCLIP_CHILDREN)
+#define PF_STYLE (wxMAXIMIZE_BOX | wxMINIMIZE_BOX | wxRESIZE_BORDER | wxSYSTEM_MENU | wxCAPTION | wxCLOSE_BOX | wxCLIP_CHILDREN)
 
 PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
     : wxFrame(frame,-1, _("Panorama preview"), wxDefaultPosition, wxDefaultSize,
@@ -347,9 +347,9 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
 
     // set the minimize icon
 #ifdef __WXMSW__
-    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/icon.ico"),wxBITMAP_TYPE_ICO);
+    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/hugin.ico"),wxBITMAP_TYPE_ICO);
 #else
-    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/icon.png"),wxBITMAP_TYPE_PNG);
+    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/hugin.png"),wxBITMAP_TYPE_PNG);
 #endif
     SetIcon(myIcon);
 
@@ -376,7 +376,7 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
     wxAcceleratorEntry entries[3];
     entries[0].Set(wxACCEL_NORMAL,WXK_F11,ID_FULL_SCREEN);
     entries[1].Set(wxACCEL_CTRL,(int)'Z',ID_UNDO);
-    entries[2].Set(wxACCEL_CTRL,(int)'Y',ID_REDO);
+    entries[2].Set(wxACCEL_CTRL,(int)'R',ID_REDO);
     wxAcceleratorTable accel(3, entries);
     SetAcceleratorTable(accel);
 #ifdef __WXGTK__
@@ -494,7 +494,7 @@ void PreviewFrame::panoramaChanged(Panorama &pano)
         assert((int) params.size() == nParam);
         for (int i=0; i < nParam; i++) {
             wxString val = wxString(doubleToString(params[i],1).c_str(), wxConvLocal);
-            m_projParamTextCtrl[i]->SetValue(wxString(val.c_str(), wxConvLocal));
+            m_projParamTextCtrl[i]->SetValue(wxString(val.wc_str(), wxConvLocal));
             m_projParamSlider[i]->SetValue(utils::roundi(params[i]));
         }
     }
@@ -972,12 +972,12 @@ void PreviewFrame::OnFullScreen(wxCommandEvent & e)
 void PreviewFrame::OnUndo(wxCommandEvent &e)
 {
     wxCommandEvent dummy(wxEVT_COMMAND_MENU_SELECTED, XRCID("ID_EDITUNDO"));
-    m_parent->AddPendingEvent(dummy);
+    m_parent->GetEventHandler()->AddPendingEvent(dummy);
 };
 
 void PreviewFrame::OnRedo(wxCommandEvent &e)
 {
     wxCommandEvent dummy(wxEVT_COMMAND_MENU_SELECTED, XRCID("ID_EDITREDO"));
-    m_parent->AddPendingEvent(dummy);
+    m_parent->GetEventHandler()->AddPendingEvent(dummy);
 };
 
