@@ -3,7 +3,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: PreviewFrame.h 4613 2009-10-12 05:35:27Z tmodes $
+ *  $Id$
  *
  *  This is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -70,12 +70,18 @@ protected:
     void OnDefaultExposure( wxCommandEvent & e );
     void OnDecreaseExposure( wxSpinEvent & e );
     void OnIncreaseExposure( wxSpinEvent & e );
+    /** event handler for reset projection parameters */
+    void OnProjParameterReset(wxCommandEvent & e);
 
     void OnBlendChoice(wxCommandEvent & e);
     void OnProjectionChoice(wxCommandEvent & e);
     void OnOutputChoice(wxCommandEvent & e);
     /** event handler for full screen */
     void OnFullScreen(wxCommandEvent &e);
+    /** event handler for undo */
+    void OnUndo(wxCommandEvent &e);
+    /** event handler for redo */
+    void OnRedo(wxCommandEvent &e);
 
     // update the panorama display
     void updatePano();
