@@ -30,6 +30,13 @@
 #include "MainFrame.h"
 
 #include <wx/platform.h>
+
+//multitexture feature requires glew on some systems
+#if !defined Hugin_shared || !defined _WINDOWS
+#define GLEW_STATIC
+#endif
+#include <GL/glew.h>
+
 #ifdef __WXMAC__
 #include <OpenGL/gl.h>
 #include <OpenGL/glu.h>
@@ -107,6 +114,9 @@ PreviewIdentifyTool::PreviewIdentifyTool(PreviewToolHelper *helper,
                           GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, rect_tex_data);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                         GL_LINEAR_MIPMAP_LINEAR);
+        // clamp texture so it won't wrap over the border.
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
     }
     {
         // the circular one should look similar to the rectangle one, but we
@@ -147,6 +157,9 @@ PreviewIdentifyTool::PreviewIdentifyTool(PreviewToolHelper *helper,
                           GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, circle_tex_data);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                         GL_LINEAR_MIPMAP_LINEAR);
+        // clamp texture so it won't wrap over the border.
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
     }
 }
 PreviewIdentifyTool::~PreviewIdentifyTool()
@@ -265,6 +278,13 @@ void PreviewIdentifyTool::AfterDrawImagesEvent()
     unsigned int image_counter = 0;
     for (it = image_set.rbegin(); it != image_set.rend(); it++)
     {
+        // Use the mask to alter the shape of the identification boxes, but
+        // replace the normal image texture with the identification box itself.
+        if (helper->GetViewStatePtr()->GetSupportMultiTexture())
+        {
+            helper->GetViewStatePtr()->GetTextureManager()->BindTexture(*it);
+            glActiveTexture(GL_TEXTURE0);
+        }
         // we want to shift the texture so it lines up with the cropped region.
         glPushMatrix();
         HuginBase::SrcPanoImage *src = helper->GetViewStatePtr()->
