@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: PreviewPanel.cpp 4921 2010-01-20 06:21:35Z tmodes $
+ *  $Id$
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -147,6 +147,11 @@ void PreviewPanel::panoramaChanged(Panorama &pano)
         DEBUG_DEBUG("projection parameters changed");
         dirty = true;
     }
+    if (newOpts.outputExposureValue != opts.outputExposureValue )
+    {
+        DEBUG_DEBUG("output exposure value changed");
+        dirty = true;
+    };
 
     opts = newOpts;
     if (dirty) {
