@@ -73,6 +73,15 @@
 
 #define HUGIN_LENS_ASSUME_SIMILAR             1l
 
+// project naming convention
+#define HUGIN_PROJECT_NAMING_CONVENTION       0l
+
+// smart undo
+#define HUGIN_SMART_UNDO                      0l
+
+// show hints in fast preview window
+#define HUGIN_SHOW_PROJECTION_HINTS           1l
+
 // Exiftool
 #define HUGIN_EXIFTOOL_COPY_ARGS                   "-ImageDescription -Make -Model -Artist -WhitePoint -Copyright -GPS:all -DateTimeOriginal -CreateDate -UserComment -ColorSpace -OwnerName -SerialNumber"
 
