@@ -56,9 +56,9 @@
 #include "base_wx/huginConfig.h"
 #ifdef __WXMSW__
 #include "wx/dir.h"
+#include "wx/cshelp.h"
 #endif
 
-
 #include <tiffio.h>
 
 #include "AboutDialog.h"
@@ -77,17 +77,31 @@ bool str2double(wxString s, double & d)
 
 wxString getDefaultProjectName(const Panorama & pano)
 {
-    if (pano.getNrOfImages() > 0) {
-        
-        wxString first_img(stripExtension(stripPath(pano.getImage(0).getFilename())).c_str(), HUGIN_CONV_FILENAME);
-        wxString last_img(stripExtension(stripPath(pano.getImage(pano.getNrOfImages()-1).getFilename())).c_str(), HUGIN_CONV_FILENAME);
-        return first_img + wxT("-") + last_img;
-    } else {
-        return wxString(wxT("pano"));
+    bool t = (wxConfigBase::Get()->Read(wxT("ProjectNamingConvention"), HUGIN_PROJECT_NAMING_CONVENTION) != 0);
+    if(t){
+        if (pano.getNrOfImages() > 0) {
+            wxString first_img(hugin_utils::getFolder(pano.getImage(0).getFilename()).c_str(), HUGIN_CONV_FILENAME);
+            return first_img;
+        } else {
+            return wxString(wxT("pano"));
+        }
+    }else{
+        if (pano.getNrOfImages() > 0) {
+
+            wxString first_img(stripExtension(stripPath(pano.getImage(0).getFilename())).c_str(), HUGIN_CONV_FILENAME);
+            wxString last_img(stripExtension(stripPath(pano.getImage(pano.getNrOfImages()-1).getFilename())).c_str(), HUGIN_CONV_FILENAME);
+            return first_img + wxT("-") + last_img;
+        } else {
+            return wxString(wxT("pano"));
+        }
     }
 }
 
+DEFINE_EVENT_TYPE( EVT_IMAGE_READY )
 
+BEGIN_EVENT_TABLE(huginApp, wxApp)
+    EVT_IMAGE_READY2(-1, huginApp::relayImageLoaded)
+END_EVENT_TABLE()
 
 // make wxwindows use this class as the main application
 IMPLEMENT_APP(huginApp)
@@ -116,6 +130,9 @@ bool huginApp::OnInit()
 {
     DEBUG_TRACE("=========================== huginApp::OnInit() begin ===================");
     SetAppName(wxT("hugin"));
+    
+    // Connect to ImageCache: we need to tell it when it is safe to handle UI events.
+    ImageCache::getInstance().asyncLoadCompleteSignal = &huginApp::imageLoadedAsync;
 
 #ifdef __WXMAC__
     // do not use the native list control on OSX (it is very slow with the control point list window)
@@ -130,6 +147,10 @@ bool huginApp::OnInit()
 
 
 #if defined __WXMSW__
+    // initialize help provider
+    wxHelpControllerHelpProvider* provider = new wxHelpControllerHelpProvider;
+    wxHelpProvider::Set(provider);
+
     wxString huginExeDir = getExePath(argv[0]);
 
     wxString huginRoot;
@@ -270,6 +291,8 @@ bool huginApp::OnInit()
     // setup main frame size, after it has been created.
     RestoreFramePosition(frame, wxT("MainFrame"));
 #ifdef __WXMSW__
+    provider->SetHelpController(&frame->GetHelpController());
+    frame->GetHelpController().Initialize(m_xrcPrefix+wxT("data/hugin_help_en_EN.chm"));
     frame->SendSizeEvent();
 #endif
 
@@ -444,6 +467,30 @@ huginApp * huginApp::Get()
     }
 }
 
+MainFrame* huginApp::getMainFrame()
+{
+    if (m_this) {
+        return m_this->frame;
+    } else {
+        return 0;
+    }
+}
+
+void huginApp::relayImageLoaded(ImageReadyEvent & event)
+{
+    ImageCache::getInstance().postEvent(event.request, event.entry);
+}
+
+void huginApp::imageLoadedAsync(ImageCache::RequestPtr request, ImageCache::EntryPtr entry)
+{
+    ImageReadyEvent event(request, entry);
+    // AddPendingEvent adds the event to the event queue and returns without
+    // processing it. This is necessary since we are probably not in the
+    // UI thread, but the event handler must be run in the UI thread since it
+    // could update image views.
+    Get()->AddPendingEvent(event);
+}
+
 #ifdef __WXMAC__
 void huginApp::MacOpenFile(const wxString &fileName)
 {
@@ -549,3 +596,29 @@ void StoreFramePosition(wxTopLevelWindow * frame, const wxString & basename)
     }
 #endif
 }
+
+IMPLEMENT_DYNAMIC_CLASS(MyHTML, wxHtmlWindow)
+
+ BEGIN_EVENT_TABLE(MyHTML, wxHtmlWindow)
+ // Add event tables here
+ END_EVENT_TABLE()
+
+ bool MyHTML::Create(wxWindow * parent, wxWindowID id,
+                 const wxPoint & pos,
+                 const wxSize & size,
+                 long style)
+ {
+   SetParent(parent);
+   wxXmlResource::Get()->LoadPanel(this, GetParent(), _T("dedication_html"));
+   GetSizer()->Fit(this);
+
+   return true;
+ }
+
+void MyHTML::OnLinkClicked(const wxHtmlLinkInfo& link)
+{
+  if (link.GetHref().StartsWith(_T("http://")))
+    wxLaunchDefaultBrowser(link.GetHref());
+  else
+    wxHtmlWindow::OnLinkClicked(link);
+}
