@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: hugin_math.h 4428 2009-09-15 15:45:47Z yuv $
+ *  $Id$
  *
  *  This is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -27,6 +27,7 @@
 #ifndef _HUGIN_MATH_HUGIN_MATH_H
 #define _HUGIN_MATH_HUGIN_MATH_H
 
+#include <hugin_shared.h>
 #include <cmath>
 #include <math.h>
 #include <iostream>
@@ -111,7 +112,7 @@ namespace hugin_utils
 
     // a simple point class
     template <class T>
-    struct TDiff2D
+    struct IMPEX TDiff2D
     {
         TDiff2D()
             : x(0), y(0)
@@ -149,11 +150,31 @@ namespace hugin_utils
                 y = y*val;
                 return *this;
             }
+        
+        TDiff2D operator*(double val)
+            {
+                TDiff2D<T> result;
+                result.x = x * val;
+                result.y = y * val;
+                return result;
+            }
 
         vigra::Diff2D toDiff2D() const
             {
                 return vigra::Diff2D(roundi(x), roundi(y));
             }
+        
+        /// Return square of the distance to another point 
+        T squareDistance(TDiff2D<T> other) const
+            {
+                return (other - *this).squareLength();
+            }
+        
+        /// Return the square of the length of the vector
+        T squareLength() const
+            {
+                return x*x + y*y;
+            }
 
         double x,y;
     };
