@@ -5,7 +5,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: Stitcher.h 3307 2008-08-11 22:29:29Z dangelo $
+ *  $Id$
  *
  *  This is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -1181,7 +1181,7 @@ static void stitchPanoIntern(const PanoramaData & pano,
  * @todo do not keep complete output image in memory
  *
  */
-void stitchPanorama(const PanoramaData & pano,
+IMPEX void stitchPanorama(const PanoramaData & pano,
 		            const PanoramaOptions & opts,
 		            AppBase::MultiProgressDisplay & progress,
 		            const std::string & basename,
