@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: PanoramaOptions.cpp 4970 2010-02-08 21:20:35Z brunopostle $
+ *  $Id$
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -22,6 +22,29 @@
  *  License along with this software; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
+ Revised 16JAN2010 by TKSharpless 
+ to support default projection  parameters and dynamic FOV limits.
+ libpano APIs
+   panoProjectionFeaturesQuery( proj, &features ) returns constant
+		values, including min, max, and default parameter values 
+		and absolute max FOV limits.
+   queryFOVLimits(proj, &parms, &fovs ) returns dynamic hFOV and vFOV 
+		limits calculated for a given set of projection parameters.
+
+  A PanoramaOptions holds a copy of the pano_projection_features from
+  panoProjectionFeaturesQuery().  Original implementation updated
+  that oftener than necessary (it can change only when the projection
+  does) possibly with the hope of tracking dynamic changes.  
+
+  Revised implementation loads the full pano_projection_features only 
+  when projection changes, and updates its max FOV values only when 
+  parameter values change.  Member fns getMaxHFOV() and getMaxVFOV() 
+  now return those local values.  
+ 
+  setProjection() now posts default projection parameters that come 
+  from libpano in the pano_projection_features block.  The default 
+  projection parameter values formerly set here are now in libpano 
+  (queryfeature.c)
  */
 
 
@@ -149,62 +172,19 @@ void PanoramaOptions::printScriptLine(std::ostream & o, bool forPTOptimizer) con
 
 void PanoramaOptions::setProjection(ProjectionFormat f)
 {
-    if ((int) f >= panoProjectionFormatCount()) {
+  // post new projection type
+   if ((int) f >= panoProjectionFormatCount()) {
         // reset to equirect if this projection is not known
         f = EQUIRECTANGULAR;
     }
-    // only try to keep the FOV if calculations are implemented for
-    // both projections
-    if (fovCalcSupported(m_projectionFormat) && fovCalcSupported(f)) 
-    {
-        // keep old fov
-        // correct image width and height
-        PanoramaOptions copy(*this);
-        copy.m_projectionFormat = f;
-        double hfov = std::min(getHFOV(), copy.getMaxHFOV());
-        double vfov = std::min(getVFOV(), copy.getMaxVFOV());
-        setHFOV(hfov, false);
-        setVFOV(vfov);
-        panoProjectionFeaturesQuery(f, &m_projFeatures);
-        m_projectionFormat = f;
-        m_projectionParams.resize(m_projFeatures.numberOfParameters);
-        if (m_projFeatures.numberOfParameters) {
-            if (f == ALBERS_EQUAL_AREA_CONIC) {
-                m_projectionParams[0] = 0;
-                m_projectionParams[1] = 60;
-            };
-			if (f == BIPLANE)
-			{
-				m_projectionParams[0] = 45;
-			};
-			if (f == TRIPLANE)
-			{
-				m_projectionParams[0] = 60;
-			};
-        }
-        setHFOV(hfov, false);
-        setVFOV(vfov);
-    } else {
-        m_projectionFormat = f;
-        panoProjectionFeaturesQuery(f, &m_projFeatures);
-        m_projectionParams.resize(m_projFeatures.numberOfParameters);
-        if (m_projFeatures.numberOfParameters) {
-            if (f == ALBERS_EQUAL_AREA_CONIC) {
-                m_projectionParams[0] = 0;
-                m_projectionParams[1] = 60;
-            };
-			if (f == BIPLANE)
-			{
-				m_projectionParams[0] = 45;
-			};
-			if (f == TRIPLANE)
-			{
-				m_projectionParams[0] = 60;
-			};
-        }
-        double hfov = std::min(getHFOV(), getMaxHFOV());
-        setHFOV(hfov, false);
-    }
+	m_projectionFormat = f;
+
+	/* Load constant features of the new projection */
+	panoProjectionFeaturesQuery(f, &m_projFeatures);
+	/* post default projection parameters and corresponding FOV limits */
+    m_projectionParams.resize(m_projFeatures.numberOfParameters);
+    // reset projection parameters to default, determine also new fov limits, clip current fovs...
+    resetProjectionParameters();
 }
 
 
@@ -224,19 +204,36 @@ void PanoramaOptions::setProjectionParameters(const std::vector<double> & params
             }
         }
     }
+	/* get dynamic FOV limits corresponding to the new parameters,
+	   clip current fovs to those limits, and post the results
+	*/
+    double parms[PANO_PROJECTION_MAX_PARMS];
+	double fovs[2];
+	int i;
+	for( i = 0; i < m_projFeatures.numberOfParameters; i++){
+		parms[i] = m_projectionParams[i];
+	}
+	if( queryFOVLimits((int)m_projectionFormat, parms, fovs )){
+		m_projFeatures.maxHFOV = fovs[0];
+		m_projFeatures.maxVFOV = fovs[1];
+	}
+	setHFOV( m_hfov, false );
+	setVFOV( getVFOV() );
 }
 
+void PanoramaOptions::resetProjectionParameters()
+{
+    std::vector<double> defParam(m_projFeatures.numberOfParameters);
+    for(int i = 0; i < m_projFeatures.numberOfParameters; i++)
+    {
+        defParam[i] = m_projFeatures.parm[i].defValue;
+    };
+    setProjectionParameters(defParam);
+};
+
 bool PanoramaOptions::fovCalcSupported(ProjectionFormat f) const
 {
-    /*
-    // should be like this but does not return BIPLANE, TRIPLANE, etc.
-     pano_projection_features pfeat;
-     if (panoProjectionFeaturesQuery((int) m_projectionFormat, &pfeat)) {
-         return pfeat.maxVFOV <=180;
-     } else {
-         return false;
-     }
-     */
+    /* Ideally this attribute should come from libpano */
     return ( f == RECTILINEAR
              || f == CYLINDRICAL
              || f == EQUIRECTANGULAR
@@ -247,7 +244,9 @@ bool PanoramaOptions::fovCalcSupported(ProjectionFormat f) const
              || f == ARCHITECTURAL
              || f == EQUI_PANINI
 			 || f == BIPLANE
-			 || f == TRIPLANE);
+			 || f == TRIPLANE
+             || f == GENERAL_PANINI
+			 );
 }
 
 
@@ -308,7 +307,7 @@ void PanoramaOptions::setHeight(unsigned int h)
         m_roi &= vigra::Rect2D(m_size);
     }
 
-    DEBUG_DEBUG(" HFOV: " << m_hfov << " size: " << m_size << " roi:" << m_roi << "  => vfov: " << getVFOV());
+    DEBUG_DEBUG(" HFOV: " << m_hfov << " size: " << m_size << " roi:" << m_roi << "  => vfov: " << getVFOV() );
 }
 
 void PanoramaOptions::setHFOV(double h, bool keepView)
@@ -429,45 +428,10 @@ double PanoramaOptions::getVFOV() const
     }
     */
     DEBUG_DEBUG(" HFOV: " << m_hfov << " size: " << m_size << " roi: " << m_roi << "  => vfov: " << VFOV);
+
     return VFOV;
 }
 
-double PanoramaOptions::getMaxHFOV() const
-{
-	double angle;
-    pano_projection_features pfeat;
-    if (panoProjectionFeaturesQuery((int)m_projectionFormat, &pfeat)) {
-		switch(m_projectionFormat)
-		{
-			case BIPLANE:
-				if (m_projectionParams.size()>0)
-					angle = m_projectionParams[0];
-				else
-					angle = 0;
-				return angle + 179;
-			case TRIPLANE:
-				if (m_projectionParams.size()>0)
-					angle = m_projectionParams[0];
-				else
-					angle = 0;
-				return 2 * angle + 179;
-			default:
-		return pfeat.maxHFOV;
-		}
-    } else {
-        return 360;
-    }
-}
-
-double PanoramaOptions::getMaxVFOV() const
-{
-    pano_projection_features pfeat;
-    if (panoProjectionFeaturesQuery((int) m_projectionFormat, &pfeat)) {
-        return pfeat.maxVFOV;
-    } else {
-        return 180;
-    }
-}
 
 DestPanoImage PanoramaOptions::getDestImage() const
 {
