@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: PanoToolsInterface.cpp 4357 2009-09-10 04:23:50Z yuv $
+ *  $Id$
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -36,6 +36,7 @@
 #include <hugin_utils/stl_utils.h>
 #include <panodata/PanoImage.h>
 #include <panodata/PanoramaData.h>
+#include <panodata/StandardImageVariableGroups.h>
 
 // This function is not defined in the pano13 headers
 extern "C" {
@@ -77,14 +78,14 @@ void Transform::updatePTData(const vigra::Diff2D &srcSize,
 void Transform::createTransform(const PanoramaData & pano, unsigned int imgNr,
                                 const PanoramaOptions & dest, vigra::Diff2D srcSize)
 {
-    const PanoImage & img = pano.getImage(imgNr);
+    const SrcPanoImage & img = pano.getImage(imgNr);
     if (srcSize.x == 0 && srcSize.y == 0) {
         srcSize.x = img.getWidth();
         srcSize.y = img.getHeight();
     }
     createTransform(srcSize,
                     pano.getImageVariables(imgNr),
-                    pano.getLens(img.getLensNr()).getProjection(),
+                    (Lens::LensProjectionFormat)pano.getImage(imgNr).getProjection(),
                     vigra::Diff2D(dest.getWidth(), dest.getHeight()),
                     dest.getProjection(), 
                     dest.getProjectionParameters(),
@@ -109,6 +110,10 @@ void Transform::createInvTransform(const SrcPanoImage & src, const PanoramaOptio
     vars.insert(make_pair(std::string("p"), Variable("p", src.getPitch())));
     vars.insert(make_pair(std::string("y"), Variable("y", src.getYaw())));
 
+    vars.insert(make_pair(std::string("TrX"), Variable("TrX", src.getX())));
+    vars.insert(make_pair(std::string("TrY"), Variable("TrY", src.getY())));
+    vars.insert(make_pair(std::string("TrZ"), Variable("TrZ", src.getZ())));
+
     createInvTransform(src.getSize(),
                        vars,
                        (Lens::LensProjectionFormat) src.getProjection(),
@@ -137,6 +142,10 @@ void Transform::createTransform(const SrcPanoImage & src, const PanoramaOptions
     vars.insert(make_pair(std::string("p"), Variable("p", src.getPitch())));
     vars.insert(make_pair(std::string("y"), Variable("y", src.getYaw())));
 
+    vars.insert(make_pair(std::string("TrX"), Variable("TrX", src.getX())));
+    vars.insert(make_pair(std::string("TrY"), Variable("TrY", src.getY())));
+    vars.insert(make_pair(std::string("TrZ"), Variable("TrZ", src.getZ())));
+
     createTransform(src.getSize(),
                     vars,
                     (Lens::LensProjectionFormat) src.getProjection(),
@@ -181,14 +190,14 @@ void Transform::createTransform(const vigra::Diff2D & srcSize,
 void Transform::createInvTransform(const PanoramaData & pano, unsigned int imgNr,
                                    const PanoramaOptions & dest, vigra::Diff2D srcSize)
 {
-    const PanoImage & img = pano.getImage(imgNr);
+    const SrcPanoImage & img = pano.getImage(imgNr);
     if (srcSize.x == 0 && srcSize.y == 0) {
         srcSize.x = img.getWidth();
         srcSize.y = img.getHeight();
     }
     createInvTransform(srcSize,
                        pano.getImageVariables(imgNr),
-                       pano.getLens(img.getLensNr()).getProjection(),
+                       (Lens::LensProjectionFormat)pano.getImage(imgNr).getProjection(),
                        vigra::Diff2D(dest.getWidth(), dest.getHeight()),
                        dest.getProjection(),
                        dest.getProjectionParameters(),
@@ -257,8 +266,20 @@ bool Transform::transformImgCoord(double & x_dest, double & y_dest,
     
     void * params= (void *) (&m_stack);
     bool ok = execute_stack_new(x_src, y_src, &x_dest, &y_dest, params) != 0;
-    x_dest += m_destTX - 0.5;
-    y_dest += m_destTY - 0.5;
+    if(ok)
+    {
+        x_dest += m_destTX - 0.5;
+        y_dest += m_destTY - 0.5;
+    }
+    else
+    {
+        // if the image coordinates could not transformed, e.g. this can happen 
+        // with non-zero translation parameters, set the result to a point
+        // outside the image (-1,-1)
+        // better solution would be to evaluate the return value of transformImgCoord
+        x_dest = -1;
+        y_dest = -1;
+    };
     return ok;
 }
 
@@ -325,12 +346,13 @@ bool AlignInfoWrap::setInfo(const PanoramaData & pano)
 
     int optVarCounter=0;
     // be careful. linked variables should not be specified multiple times.
-    std::vector<std::set<std::string> > linkvars(pano.getNrOfLenses());
+    ConstStandardImageVariableGroups variable_groups(pano);
+    std::vector<std::set<std::string> > linkvars(variable_groups.getLenses().getNumberOfParts());
 
     for (unsigned imgNr = 0; imgNr < variables.size(); imgNr++)
     {
-        unsigned int lensNr = pano.getImage(imgNr).getLensNr();
-        const Lens & lens = pano.getLens(lensNr);
+        unsigned int lensNr = variable_groups.getLenses().getPartNumber(imgNr);
+        const Lens & lens = variable_groups.getLens(lensNr);
         const std::set<std::string> & optvar = optvec[imgNr];
         for (std::set<std::string>::const_iterator sit = optvar.begin();
              sit != optvar.end(); ++sit )
@@ -390,11 +412,11 @@ bool AlignInfoWrap::setInfo(const PanoramaData & pano)
     std::map<unsigned int, unsigned int> linkAnchors;
     for(unsigned imgNr=0; (int)imgNr< gl.numIm; imgNr++)
     {
-        const PanoImage & pimg = pano.getImage(imgNr);
+        const SrcPanoImage & pimg = pano.getImage(imgNr);
         const VariableMap & vars = variables[imgNr];
-        unsigned lensNr = pimg.getLensNr();
+        unsigned lensNr = variable_groups.getLenses().getPartNumber(imgNr);
         //pano.getImageVariables(*it);
-        const Lens & lens = pano.getLens(lensNr);
+        const Lens & lens = variable_groups.getLensForImage(imgNr);
 
         // set the image information, with pointer to dummy image data
         setFullImage(gl.im[imgNr],
@@ -484,6 +506,10 @@ VariableMapVector GetAlignInfoVariables(const AlignInfo& gl)
     if (gl.im) {
         for (int i = 0; i < gl.numIm; i++) {
             VariableMap vars;
+            vars.insert(make_pair(std::string("TrX"), Variable("TrX", gl.im[i].cP.trans_x)));
+            vars.insert(make_pair(std::string("TrY"), Variable("TrY", gl.im[i].cP.trans_y)));
+            vars.insert(make_pair(std::string("TrZ"), Variable("TrZ", gl.im[i].cP.trans_z)));
+
             vars.insert(make_pair(std::string("v"), Variable("v", gl.im[i].hfov)));
             vars.insert(make_pair(std::string("y"), Variable("y", gl.im[i].yaw)));
             vars.insert(make_pair(std::string("r"), Variable("r", gl.im[i].roll)));
@@ -593,6 +619,16 @@ void initCPrefs(cPrefs & p, const VariableMap &vars)
         p.horizontal = FALSE;
         p.horizontal_params[0] = p.horizontal_params[1] = p.horizontal_params[2] = 0;
     }
+
+    p.trans_x = const_map_get(vars,"TrX").getValue();
+    p.trans_y = const_map_get(vars,"TrY").getValue();
+    p.trans_z = const_map_get(vars,"TrZ").getValue();
+    if (p.trans_x != 0 || p.trans_y != 0 || p.trans_z != 0) {
+	p.trans = TRUE;
+    } else {
+	p.trans = FALSE;
+    }
+
     // FIXME add shear parameters
     val = const_map_get(vars, "g").getValue();
     double val2 = const_map_get(vars, "t").getValue();
@@ -638,21 +674,30 @@ void setFullImage(Image & image, vigra::Diff2D size,
 
     image.dataformat = _RGB;
     switch (format) {
-    case Lens::RECTILINEAR:
+    case BaseSrcPanoImage::RECTILINEAR:
         image.format = _rectilinear;
         break;
-    case Lens::PANORAMIC:
+    case BaseSrcPanoImage::PANORAMIC:
         image.format = _panorama;
         break;
-    case Lens::CIRCULAR_FISHEYE:
+    case BaseSrcPanoImage::CIRCULAR_FISHEYE:
         image.format = _fisheye_circ;
         break;
-    case Lens::FULL_FRAME_FISHEYE:
+    case BaseSrcPanoImage::FULL_FRAME_FISHEYE:
         image.format = _fisheye_ff;
         break;
-    case Lens::EQUIRECTANGULAR:
+    case BaseSrcPanoImage::EQUIRECTANGULAR:
         image.format = _equirectangular;
         break;
+    case BaseSrcPanoImage::FISHEYE_ORTHOGRAPHIC:
+        image.format = _orthographic;
+        break;
+    case BaseSrcPanoImage::FISHEYE_STEREOGRAPHIC:
+        image.format = _stereographic;
+        break;
+    case BaseSrcPanoImage::FISHEYE_EQUISOLID:
+        image.format = _equisolid;
+        break;
     }
     image.hfov = const_map_get(vars,"v").getValue();
     image.yaw = const_map_get(vars,"y").getValue();
@@ -666,9 +711,6 @@ void setFullImage(Image & image, vigra::Diff2D size,
 
     // no name
     image.name[0]=0;
-    image.yaw = const_map_get(vars,"y").getValue();
-    image.yaw = const_map_get(vars,"y").getValue();
-
 
     image.selection.top = 0;
     image.selection.left = 0;
