@@ -5,7 +5,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: PanoToolsOptimizerWrapper.h 4387 2009-09-12 19:37:13Z gpatters1 $
+ *  $Id$
  *
  *  This is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -55,7 +55,7 @@ namespace PTools
      * @return 0:good, 1:parser error, 2: parameter error
      *
      */
-    unsigned int optimize(PanoramaData & pano,
+    IMPEX unsigned int optimize(PanoramaData & pano,
                   const char * script = 0);
 
 } // namespace
